#
#
# adapter.py - BlueZ adapter interface interface
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import dbus
import bluetooth

import time
import datetime

from blueman.Main.address_cache import address_cache

import blueman.Globals as Globals
_ = Globals._
ngettext = Globals.ngettext

class adapter():
	
	def __init__(self, device):
		self.device = device
		self.bus = dbus.SystemBus();
		self.obj = self.bus.get_object('org.bluez', '/org/bluez/'+device)
		self.adapter = dbus.Interface(self.obj, 'org.bluez.Adapter')
		
		self.bus.add_signal_receiver(self.on_trust_added, 'TrustAdded', 'org.bluez.Adapter', 'org.bluez', '/org/bluez/'+device)
		self.bus.add_signal_receiver(self.on_trust_removed, 'TrustRemoved', 'org.bluez.Adapter', 'org.bluez', '/org/bluez/'+device)

	
	# signal handlers ##############################################
	def on_trust_added(self, address):
		rowid = Globals.main_list.get_conditional(address=address)
		if len(rowid)>0:

			Globals.main_list.set_image(rowid[0], "trusted_pb", "trust.png")
			Globals.main_list.set(rowid[0], trusted=True)
				
	def on_trust_removed(self, address):
		rowid = Globals.main_list.get_conditional(address=address)
		if len(rowid)>0:

			Globals.main_list.set(rowid[0], trusted_pb=None)
			Globals.main_list.set(rowid[0], trusted=False)
	
	#############################################################3
	
	def ListPorts(self):
		return self.adapter.ListPorts()
		
	def GetRemoteServices(self,addr,name=None,uuid=None):
		
		return bluetooth.find_service(name=name, uuid=name, address=addr)	
	
	def GetRemoteServiceClasses(self,addr):
		return self.adapter.GetRemoteServiceClasses(addr)
	
	def GetDiscoverableTimeout(self):
		return self.adapter.GetDiscoverableTimeout()
		
	def SetDiscoverableTimeout(self, timeout):
		self.adapter.SetDiscoverableTimeout(dbus.UInt32(timeout))

	
	#-----------local device functions--------------------#
	
	
	def ListAvailableMinorClasses(self):
		return self.adapter.ListAvailableMinorClasses()
	
	def GetAddress(self):
		return self.adapter.GetAddress();
	
	def GetMinorClass(self):
		return self.adapter.GetMinorClass();
		
	def SetMinorClass(self, minor):
		self.adapter.SetMinorClass(minor)
	
	def GetMajorClass(self):
		return self.adapter.GetMajorClass()
		

	
	def GetMode(self):
		return self.adapter.GetMode()
		
	def SetMode(self, mode):
		self.adapter.SetMode(mode)
		
	def ListAvailableModes(self):
		return self.adapter.ListAvailableModes()
		
	
	def GetAddress(self):
		return self.adapter.GetAddress()
		
	def GetVersion(self):
		return self.adapter.GetVersion()
		
	def GetRevision(self):
		return self.adapter.GetRevision()
		
	def GetCompany(self):
		return self.adapter.GetCompany()
		
	
	def GetManufacturer(self):
		return self.adapter.GetManufacturer()

			
	def GetName(self):
		return self.adapter.GetName()
		
	def SetName(self, name):
		self.adapter.SetName(name)
		
	def IsConnectable(self):
		return self.adapter.IsConnectable()
		
	def ListRemoteDevices(self):
		days = Globals.Settings.ls_days
		if int(days) == -1:
			then = datetime.datetime(2000,1,1)
		else:
			now = datetime.datetime(2000,1,1)
			now = now.today()
		
			delta_t = datetime.timedelta(int(days))
			then = now - delta_t
		

		return self.adapter.ListRecentRemoteDevices("%s-%s-%s 00:00:00 GMT" % (then.year, then.month, then.day))
		
		
	def ListConnections(self):
		return self.adapter.ListConnections()
		
	def ListBondings(self):
		return self.adapter.ListBondings()
		
	def is_valid_address(self, addr):
		return bluetooth.is_valid_address(addr)
		
	def SetTrusted(self, addr):
		return self.adapter.SetTrusted(addr)
	
	def IsTrusted(self, addr):
		return self.adapter.IsTrusted(addr)
	
	def RemoveTrust(self, addr):
		return self.adapter.RemoveTrust(addr)
		
	def DisconnectRemoteDevice(self, addr, *args, **kwargs):
		return self.adapter.DisconnectRemoteDevice(addr, *args, **kwargs)
		

	def ListConnections(self):
		return self.adapter.ListConnections()
		
		
#-----------remote device functions--------------------#

	
	def CancelBondingProcess(self, addr):

		handle = self.adapter.RemoveBonding(addr)
		return True
		
			
	def HasBonding(self,addr):
		if(self.adapter.HasBonding(addr)):
			return True
		else:
			return False

			
	def GetRemoteVersion(self, addr):
		return self.adapter.GetRemoteVersion(addr)
		

		
	def IsConnected(self, addr):
		return self.adapter.IsConnected(addr)
		

	def GetRemoteName(self,addr):
		#print "name request for %s" % addr
		cache = address_cache()
		cname = cache.pull(addr)
		if cname:
			Globals.dbg.info("Found name for %s in cache. Resolved as %s" % (addr, cname), 8)
			#print "-------------------"
			return cname
		else:
			#print "name not in cache. Asking hcid"
			Globals.dbg.warn("Name for %s not in cache. Asking hcid" % addr, 8)
			try:
				
				name = self.adapter.GetRemoteName(addr)
				Globals.dbg.info("hcid returned %s" % addr, 8)
				cache.push(addr, name)
				#print "-------------------"
				return name
				
			
			except:
				#print "hcid doesn't know, caching 'N/A'"
				Globals.dbg.warn("hcid doesn't know, caching 'N/A'", 8)
				cache.push(addr, "N/A")
				#print "-------------------"
				return "N/A"
		

		
	def GetRemoteMinorClass(self, addr):
		return self.adapter.GetRemoteMinorClass(addr)
		
	def GetRemoteMajorClass(self, addr):

		return self.adapter.GetRemoteMajorClass(addr)
		
	def GetRemoteRevision(self, addr):
		return self.adapter.GetRemoteRevision(addr)
		
	def GetRemoteManufacturer(self, addr):
		return self.adapter.GetRemoteManufacturer(addr)
	
	def GetRemoteCompany(self, addr):
		return self.adapter.GetRemoteCompany(addr)
	
	def LastUsed(self,addr):
		return self.adapter.LastUsed(addr)

		
	def LastSeen(self, addr, friendly=True):


		if friendly:

	
			then_str = self.adapter.LastSeen(addr)
			then = datetime.datetime(*time.strptime(then_str, "%Y-%m-%d %H:%M:%S %Z")[0:6])

			now = datetime.datetime(2000,1,1)
			now = now.today()
	
			diff = now - then
	
			if diff.days == 0:
				return _("Today")
			if diff.days == 1:
				return _("Yesterday")
		
			elif diff.days > 7:
				return  ngettext("%(0)s week ago", "%(0)s weeks ago", diff.days/7) % {"0":diff.days/7}
	
			elif diff.days > 1:
				return ngettext("%(0)s day ago", "%(0)s days ago", diff.days) % {"0":diff.days}
		
		else:
			return self.adapter.LastSeen(addr)
		
	
		
	def GetRemoteAlias(self,addr):

		return self.adapter.GetRemoteAlias()

	


