#
#
# bonding.py - bluetooth bonding procedures
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import dbus

_ = Globals._

class bonding():
	
	def __init__(self, hci):
		
		self.bus = dbus.SystemBus();
		self.obj = self.bus.get_object('org.bluez', '/org/bluez/'+hci)
		self.adapter = dbus.Interface(self.obj, 'org.bluez.Adapter')
		
		self.hci = hci
		self.bus.add_signal_receiver(self.bonding_created_signal, 'BondingCreated', 'org.bluez.Adapter', 'org.bluez', None)

		self.active = False
		
	def Deinit(self):
		self.bus.remove_signal_receiver(self.bonding_created_signal, 'BondingCreated', 'org.bluez.Adapter', 'org.bluez', None)
		
		
	def bonding_created_signal(self,bdaddr):
       		self.on_bonding_finished((True,bdaddr))

	
	def Create(self,addr):
		self.active = True
		Globals.toolbar.set_bond_btn()
		self.adapter.CreateBonding(addr, reply_handler=self.BondingReply, error_handler=self.BondingError)
		return True
		
	
	def BondingReply(self):
		pass
		
	
	def BondingError(self,err):
		self.on_bonding_finished((False,err))
		

	def Remove(self,addr):
		handle = self.adapter.RemoveBonding(addr)
		rowid = Globals.main_list.get_conditional(address=addr)
		Globals.main_list.set(rowid[0], bonded=False)
		return True
	
	def on_bonding_finished(self, result):
		
		if result[0]:
			Globals.Statusbar.push(_("Bonding successful."))
			addr = result[1]
			Globals.toolbar.bond1.set_label("Unbond");
			
			if Globals.Settings.other_notifications:
				Globals.Notification.show(_("Bonding successful."), _("Bonding with %(addr)s (%(name)s) created") % {"addr":addr,"name":Globals.adapter.GetRemoteName(addr)}, im="lock_bt.png" )
			
			rowid = Globals.main_list.get_conditional(address=addr)

			if len(rowid) > 0:
				row = Globals.main_list.get(rowid[0], "bonded", "connected", "address")
			
				bonded = row["bonded"]
				connected = row["connected"]
				bdaddr = row["address"]
			

				Globals.main_list.set_image(rowid[0], "bonded_pb", "hasbonding.png")
				Globals.main_list.set(rowid[0], bonded=True)
			
		else:
			Globals.Statusbar.push(_("Bonding Failed."))
			print result[1]
			
		self.active = False
		Globals.toolbar.set_bond_btn()

