#
#
# connectivity.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import dbus
from blueman.Gui.signal_quality import signal_quality
import gobject

_ = Globals._

class connectivity( gobject.GObject ):
	__gsignals__ = {
		"device-connected": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
		"device-disconnected": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,))	
	}
	
	def __init__(self):
		gobject.GObject.__init__(self)
		self.bus = dbus.SystemBus();
		
		self.bus.add_signal_receiver(self.device_connected_signal, 'RemoteDeviceConnected', 'org.bluez.Adapter', 'org.bluez', None)
		self.bus.add_signal_receiver(self.device_disconnected_signal, 'RemoteDeviceDisconnected', 'org.bluez.Adapter', 'org.bluez', None)	
		
		self.connections = list(Globals.adapter.ListConnections())
		
	
	def Deinit(self):
		self.bus.remove_signal_receiver(self.device_connected_signal, 'RemoteDeviceConnected', 'org.bluez.Adapter', 'org.bluez', None)
		self.bus.remove_signal_receiver(self.device_disconnected_signal, 'RemoteDeviceDisconnected', 'org.bluez.Adapter', 'org.bluez', None)
	

	
	def device_connected_signal(self, addr):
		
		if not addr in self.connections:
			self.connections.append(addr)
			self.connection((True, addr))
		
		self.emit("device-connected", addr)
		
	def device_disconnected_signal(self, addr):
		
		if addr in self.connections:
			self.connections.remove(addr)
		self.connection((False, addr))	
		self.emit("device-disconnected", addr)
	
	def add(self, address):
		address = address.upper()
		if not address in self.connections:
			self.connections.append(str(address))
		
	def remove(self, address):
		self.connections.remove(address)
	

	def connection(self,result):
		
		addr = result[1]
		if result[0]:
			if Globals.Settings.connection_notifications:
				name = Globals.adapter.GetRemoteName(addr)
				Globals.Notification.show(_("Device Connected"), _("Device %(addr)s (%(name)s) Has Connected") % {"addr": addr,"name": name}, im="con_1.png")
		else:
			if Globals.Settings.connection_notifications:
				name = Globals.adapter.GetRemoteName(addr)
				Globals.Notification.show(_("Device Disconnected"), _("Device %(addr)s (%(name)s) Has Disconnected") % {"addr": addr,"name": name}, im="con_2.png")
		
		


		rowid = Globals.main_list.get_conditional(address=addr)
		if len(rowid) > 0:
			row = Globals.main_list.get(rowid[0], "bonded", "connected", "address")
			iter = Globals.main_list.get_iter(rowid[0])
			
			bonded = row["bonded"]
			connected = row["connected"]
			bdaddr = row["address"]

			if bdaddr == addr and not connected and result[0]:
				srow = Globals.main_list.get(Globals.main_list.selected(), "address")
				if srow:
					if addr == srow["address"]:
						Globals.window.mi_disconnect.set_sensitive(True)

			
				Globals.main_list.set(iter, connected=True)
				signal_quality(addr, iter)


		
			elif bdaddr == addr and connected and not result[0]:
				srow = Globals.main_list.get(Globals.main_list.selected(), "address")
				if srow:
					if addr == srow["address"]:
						Globals.window.mi_disconnect.set_sensitive(True)
			
				Globals.main_list.set(iter, connected=False, rssi_pb=None, lq_pb=None, tpl_pb=None)
				Globals.window.mi_disconnect.set_sensitive(False)


