#
#
# blueman - main_list library
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import gtk
import os

import math
import re


import gobject

from blueman.Gui.generic_list import generic_list
from blueman.Gui.signal_quality import signal_quality, gen_img_path

_ = Globals._
ngettext = Globals.ngettext



from blueman.Gui.CellRendererPixbufTable import CellRendererPixbufTable
from blueman.Gui.PixbufTable import PixbufTable



class main_list(generic_list):
	
	def __init__(self, xml):
		self.tooltip_row = -1
		self.tooltip_col = None
		self.xml = xml
		
		cust = gtk.CellRendererText()
		cust.set_property("xalign", 0.5)

		
		data = [
			["device_pb", 'GdkPixbuf', gtk.CellRendererPixbuf(), {"pixbuf":0}, None],
			
			["main_text", str, gtk.CellRendererText(), {"markup":1}, None],
			["last_seen", str, cust, {"markup":2}, "", {"expand": True}],
			
			["address", str],
			["name", str],
			
			["rssi_pb", 'GdkPixbuf', gtk.CellRendererPixbuf(), {"pixbuf":5}, None, {"spacing": 0}],
			["lq_pb", 'GdkPixbuf', gtk.CellRendererPixbuf(), {"pixbuf":6}, None, {"spacing": 0}],
			["tpl_pb", 'GdkPixbuf', gtk.CellRendererPixbuf(), {"pixbuf":7}, None, {"spacing": 0}],
			
			["bonded_pb", 'GdkPixbuf', gtk.CellRendererPixbuf(), {"pixbuf":8}, None, {"spacing": 0}],
			["trusted_pb", 'GdkPixbuf', gtk.CellRendererPixbuf(), {"pixbuf":9}, None, {"spacing": 0}],
			
			["connected", bool],
			["bonded", bool],
			["trusted", bool],	
			
			["rssi", float],
			["lq", float],
			["tpl", float],
			#["test", 'PyObject', CellRendererPixbufTable(), {"pixbuffs":16}, None]
		
			
		]
		
		generic_list.__init__(self, data)
		
		sw = self.xml.get_widget("sw")
		sw.add(self)
		self.show()
		
		self.set_headers_visible(False)
		#main text, last seen text, address, name, device logo, rssi pb, bondedpb, connected, bonded, lq pb, tpl pb, rssi, lq, tpl, trust pb, trusted
#		self.liststore = gtk.ListStore(str,str,str,str,'GdkPixbuf', 'GdkPixbuf', 'GdkPixbuf',bool,bool, 'GdkPixbuf', 'GdkPixbuf', float, float, float, 'GdkPixbuf', bool)
#		self.treeview = self.xml.get_widget("list");
		self.set_property("has-tooltip", True)
		self.connect("query-tooltip", self.tooltip_query)

		self.connect("cursor-changed", self.on_list_cursor_changed)

#		self.treeview.set_model(self.liststore);
		self.connect("drag_data_received", self.drag_recv)
		self.connect("drag-motion", self.drag_motion)

		self.drag_dest_set(gtk.DEST_DEFAULT_ALL, [ ( "text/plain", 0, 80 )]
, gtk.gdk.ACTION_COPY)

		self.row_count = 0
		
	def drag_recv(self, widget, context, x, y, selection, targetType, time):
		path = self.get_path_at_pos(x, y)
		if path != None:
			iter = self.selected()
			file_path = selection.get_text()
			match = re.match("file://(.*)", file_path)
			if match:
				f = match.groups(1)[0]
				bdaddr = self.get(iter, "address")["address"]
				os.system("bluetooth-sendto --device=%s %s &" % (bdaddr, f))
		
	def drag_motion(self, widget, drag_context, x, y, timestamp):
		path = self.get_path_at_pos(x, y)
		if path != None:
			self.set_cursor(path[0])
		

		
	def tooltip_query(self, tw, x, y, kb, tooltip):

		#print args
		#args[4].set_text("test"+str(args[1]))
		
		path = self.get_path_at_pos(x, y)


		if path:
			if path[0][0] != self.tooltip_row or path[1] != self.tooltip_col:
				self.tooltip_row = path[0][0]
				self.tooltip_col = path[1]
				return False
			
			if path[1] == self.columns["bonded_pb"]:
				iter = self.get_iter(path[0][0])

				if self.get(iter, "bonded")["bonded"]:
					tooltip.set_markup(_("<b>Bonded</b>"))
					self.tooltip_row = path[0][0]
					self.tooltip_col = path[1]
					return True
			
			if path[1] == self.columns["trusted_pb"]:
				iter = self.get_iter(path[0][0])
				#print iter
				if self.get(iter, "trusted")["trusted"]:
					tooltip.set_markup(_("<b>Trusted</b>"))
					self.tooltip_row = path[0][0]
					self.tooltip_col = path[1]
					return True
			
			if path[1] == self.columns["tpl_pb"] or path[1] == self.columns["lq_pb"] or path[1] == self.columns["rssi_pb"]:
				iter = self.get_iter(path[0][0])
				#print iter
				dt = self.get(iter, "connected")["connected"]
				#print dt
				if dt:
					rssi = self.get(iter, "rssi")["rssi"]
					lq = self.get(iter, "lq")["lq"]
					tpl = self.get(iter, "tpl")["tpl"]
					
					if rssi < 30:
						rssi_state = _("Poor")
					
					if rssi < 40 and rssi > 30:
						rssi_state = _("Sub-optimal")

					elif rssi > 40 and rssi < 60:
						rssi_state = _("Optimal")
					
					elif rssi > 60:
						rssi_state = _("Much")
					
					elif rssi > 70:
						rssi_state = _("Too much")
						
					
					if tpl < 30:
						tpl_state = _("Low")
					
					if tpl < 40 and tpl > 30:
						tpl_state = _("Sub-optimal")

					elif tpl > 40 and rssi < 60:
						tpl_state = _("Optimal")
					
					elif tpl > 60:
						tpl_state = _("High")
					
					elif tpl > 70:
						tpl_state = _("Very High")
						
											
					tooltip.set_markup(_("<b>Connected</b>\nReceived Signal Strength: %(rssi)u%% <i>(%(rssi_state)s)</i>\nLink Quality: %(lq)u%%\nTransmit Power Level: %(tpl)u%% <i>(%(tpl_state)s)</i>") % {"rssi_state": rssi_state, "rssi":rssi, "lq":lq, "tpl":tpl, "tpl_state": tpl_state})
					self.tooltip_row = path[0][0]
					self.tooltip_col = path[1]
					return True	
		return False
	
	
	
	
	def add_device(self, address, name=None, rssi=None, **params):
		address = address.upper()
		#valid  = ["pixbuf", "devtype", "connected", "bonded", "name", "lastseen", "rssi", "trusted"]
		
		try:
			bonded = Globals.adapter.HasBonding(address)
		except:
			bonded = False
		
		try:
			trusted = Globals.adapter.IsTrusted(address)
		except:
			trusted = False
		
		try:
			devtype = Globals.adapter.GetRemoteMinorClass(address)
			
			
		except:
			devtype = _("Unknown")
		try:
			connected = Globals.adapter.IsConnected(address)
		except:
			connected = False
		
		try:	
			lastseen = Globals.adapter.LastSeen(address)
		except:
			lastseen = _("Never")
			
		if name == None:
			try:
				name = Globals.adapter.GetRemoteName(address)
			except:
				name = _("N/A")
		
		lastseen = _("<span size='8000' rise='-40000' weight='light'><b>Last Seen:</b> %s</span>") % lastseen
		
		



		if bonded:	
			bonded_pb = gtk.gdk.pixbuf_new_from_file(Globals.IMAGE_DIR + "/hasbonding.png")
		else:
			bonded_pb = None

		if not trusted:
			trusted_pb = None
			trusted = False
		else:
			trusted = True
			trusted_pb = gtk.gdk.pixbuf_new_from_file(Globals.IMAGE_DIR + "/trust.png");

		if rssi != None:
			rssi_img = gen_img_path(rssi, Globals.IMAGE_DIR + "/signal/rssi/rssi_", ".png")
			rssi_pb = gtk.gdk.pixbuf_new_from_file(rssi_img)
		else:
			rssi_pb = None

			
		if not os.path.exists(Globals.IMAGE_DIR + "/devices/%s.png" % devtype):
			pb = gtk.gdk.pixbuf_new_from_file(Globals.IMAGE_DIR + "/bluetooth.png")
		else:
			pb = gtk.gdk.pixbuf_new_from_file(Globals.IMAGE_DIR + "/devices/%s.png" % devtype)
			
		devtype = devtype.capitalize()
		main_text = self.get_devinfo_string(address, name, devtype)
		iter = self.add(
				 device_pb=pb,
				 main_text=main_text,
				 last_seen=lastseen,
				 address=address.upper(),
				 name=name,
				 rssi_pb=rssi_pb,
				 lq_pb=None,
				 tpl_pb=None,
				 bonded_pb=bonded_pb,
				 trusted_pb=trusted_pb,
				 connected=connected,
				 bonded=bonded,
				 trusted=trusted,
				 rssi=0,
				 lq=0,
				 tpl=0
				)
		self.row_count+=1
				
		if connected:
			signal_quality(address, iter)
			 
			 
	def add_from_list(self, list):
		self.reset()

		for i in range(len(list)):
			self.add_device(list[i])
			
		Globals.Statusbar.push(ngettext("Listed %s device", "Listed %s devices", self.row_count) % self.row_count)
 


	def set_main_text(self, address, name, devtype=None, row=None):
		if row == None:
			rows = self.get_conditional(address=address)
			if len(rows) > 0:
				row = rows[0]
			else:
				return
		
		string = self.get_devinfo_string(address, name, devtype, row)
		
		self.set(row, main_text=string)

		

		
	
	def get_devinfo_string(self, address=None, name=None, devtype=None, row=None):
		if row != None:
			data = self.get(row, "address", "name")
			if address == None:
				address = data["address"]
			if name == None:
				name = data["name"]
				
			if devtype == None:
				devtype = Globals.adapter.GetRemoteMinorClass(address)
		return "<span size='x-large'>%(0)s</span>\n<span size='small'>%(1)s</span>\n<i>%(2)s</i>" % {"0":name, "1":devtype, "2":address}
		#return _("<b>Name:</b> %(0)s\n<b>Device type:</b> %(1)s\n<b>Address:</b> %(2)s") % {"0":name, "1":devtype, "2":address}
	

	def set_image(self, row, col, icon):
		if not icon == None:
			pb = gtk.gdk.pixbuf_new_from_file(Globals.IMAGE_DIR + "/" + icon);
		else:
			pb = None
			
		self.set(row, **{col:pb})	
	
	
	def reset(self):
		self.row_count = 0
		self.liststore.clear();
		

	def on_list_cursor_changed(self, treeview):
		
		
		row = self.get(self.selected(), "bonded", "address", "connected", "trusted")
		
		bonded = row["bonded"]
		connected = row["connected"]
		bdaddr = row["address"]
		trusted = row["trusted"]
		
		Globals.toolbar.enable_buttons()
		
		

		if not bonded:
			Globals.toolbar.bond1.set_label(_("Bond"))
		else:
			Globals.toolbar.bond1.set_label(_("Unbond"))
			
		
		if not trusted:
			Globals.window.mi_trust.get_child().set_text(_("Set Trusted"))
			im = Globals.window.mi_trust.get_image()
			im.set_from_file(Globals.IMAGE_DIR + "/trust16.png")
		else:
			Globals.window.mi_trust.get_child().set_text(_("Remove Trust"))
			im = Globals.window.mi_trust.get_image()
			im.set_from_file(Globals.IMAGE_DIR + "/trust16_g.png")
			
		if not connected:
			Globals.window.mi_disconnect.set_sensitive(False)
		else:
			Globals.window.mi_disconnect.set_sensitive(True)

