#
#
# blueman - properties module
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import gtk
import threading

from blueman.Gui.generic_dialog import generic_dialog

_ = Globals._

class properties(generic_dialog):
	
	def __init__(self):	
		generic_dialog.__init__(self, "properties.glade", "properties")
		
		self.xml.signal_autoconnect(self)

		#-- service list

		self.service_treeview = self.xml.get_widget("treeview1");

		self.service_list = gtk.ListStore(str,str)
		self.service_treeview.set_model(self.service_list)


		cell = gtk.CellRendererText()


		tvcolumn = gtk.TreeViewColumn(_('Name'))
		tvcolumn2 = gtk.TreeViewColumn(_('Channel'))

		self.service_treeview.append_column(tvcolumn)
		self.service_treeview.append_column(tvcolumn2)

		tvcolumn.pack_start(cell, True)
		tvcolumn2.pack_start(cell, True)

		tvcolumn.set_attributes(cell, markup=0)
		tvcolumn2.set_attributes(cell, markup=1)


		#---
		button = self.xml.get_widget("button3");
		button.set_sensitive(True)
		
		
		self.set_info()
		
		self.dialog.show()

	
	def on_page_changed(self, widget, *args):
		self.set_info()
	
	
	def set_info(self):

		lversion = self.xml.get_widget("label16");
		lrevision = self.xml.get_widget("label17");
		lmanufacturer = self.xml.get_widget("label18");
		lcompany = self.xml.get_widget("label19");
		lmajorclass = self.xml.get_widget("label20");
		lminorclass = self.xml.get_widget("label21")
		ldevname = self.xml.get_widget("label22")
		ldevaddress = self.xml.get_widget("label23");
		llastseen = self.xml.get_widget("label24");
		llastused = self.xml.get_widget("label25");
		lbonded = self.xml.get_widget("label26");
		lconnected = self.xml.get_widget("label27");


		addr = Globals.main_list.get(Globals.main_list.selected(), "address")["address"]
		
		
			
		try:	
			version = Globals.adapter.GetRemoteVersion(addr)
			lversion.set_label(version)
		except:
			lversion.set_label(_("Not Available"))
		
		
		try:
			connected = Globals.adapter.IsConnected(addr)
			if connected:
				lconnected.set_label(_("Yes"))
			else:
				lconnected.set_label(_("No"))
		except:
			lconnected.set_label(_("Not Available"))
		
		try:
			name = Globals.adapter.GetRemoteName(addr)
			ldevname.set_label(name)
		except:
			ldevname.set_label(_("Not Available"))
		
		try:
			minorclass = Globals.adapter.GetRemoteMinorClass(addr)
			lminorclass.set_label(minorclass)
		except:
			lminorclass.set_label(_("Not Available"))
			
			
		try:
			majorclass = Globals.adapter.GetRemoteMajorClass(addr)
			lmajorclass.set_label(majorclass)
		except:
			lmajorclass.set_label(_("Not Available"))
		
		try:
			manufacturer = Globals.adapter.GetRemoteManufacturer(addr)
			lmanufacturer.set_label(manufacturer)
		except:
			lmanufacturer.set_label(_("Not Available"))
		
		try:
			lastseen = Globals.adapter.LastSeen(addr)
			llastseen.set_label(lastseen)
		except:
			llastseen.set_label(_("Not Available"))
		
		try:
			lastused = Globals.adapter.LastUsed(addr)
			llastused.set_label(lastused)
		except:
			llastused.set_label("Not Available")
		
		try:
			bonded = Globals.adapter.HasBonding(addr)
			if bonded:
				lbonded.set_label(_("Yes"))
			else:
				lbonded.set_label(_("No"))
		except:
			lbonded.set_label(_("Not Available"))
		
		try:
			revision = Globals.adapter.GetRemoteRevision(addr)
			lrevision.set_label(revision)
		except:
			lrevision.set_label(_("Not Available"))
		
		
		try:
			company = Globals.adapter.GetRemoteCompany(addr)
			lcompany.set_label(company)
			
		except:
			
			lcompany.set_label(_("Not Available"))

		

		ldevaddress.set_label(addr)

	
	
	def set_text(self, text):
		self.label.set_text(text)

			

	
	def on_get_service_list_clicked(self, button):

		self.service_list.clear()
		addr = Globals.main_list.get(Globals.main_list.selected(), "address")["address"]
		threading.Thread(target=self.get_service_list_thread, args=(addr, self.get_service_list,)).start()
		
		button = self.xml.get_widget("button3");
		button.set_sensitive(False)
		
		

	def get_service_list_thread(self, addr,func):
		services = Globals.adapter.GetRemoteServices(addr)
		if not services:
			func(False)
		
		address = Globals.main_list.get(Globals.main_list.selected(), "address")["address"]
		if address == addr:
			func(services)			
		
		
	def get_service_list(self, services):
		try:
			button = self.xml.get_widget("button3");
			button.set_sensitive(True)
		
			if not services:
				Globals.Statusbar.push(_("Failed to get service list or no services available"))
				return 0
		

			for i in range(len(services)):
				if services[i]["name"] and services[i]["port"]:
				       	self.service_list.append([services[i]["name"],services[i]["port"]])
		except:
			pass

