#
#
# signal_quality.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
import blueman.blueman as blueman
import blueman.Globals as Globals
import gobject
import gtk

from blueman.Gui.list_service_monitor import list_service_monitor


class signal_quality:

	def __init__(self, address, row=None):
		Globals.dbg.info("Initializing signal_quality class", 6)
		list_service_monitor(address)
		
		self.list = Globals.main_list
		
		self.firstrun = True
		self.address = address
		
		self.cinfo = blueman.conn_info(address, Globals.Settings.adapter)
		
		if row == None:
			row = self.list.get_conditional(address=address)
			iter = self.list.get_iter(row[0])
			self.row = iter
			
		else:
			if type(row) == gtk.TreeIter:
				self.row = row
			else:
				self.row = self.list.get_iter(row)
		
		self.update_signal()
		gobject.timeout_add(1000, self.update_signal)
	
	
	
	
	def update_signal(self):

		if self.firstrun:
			self.list.set(self.row, rssi=-1, lq=-1, tpl=-1)
			self.firstrun = False
		if not Globals.Inquiry.active:
			
			if self.address in Globals.Connectivity.connections and self.list.liststore.iter_is_valid(self.row):
				try:
					rssi = float(self.cinfo.get_rssi())
					lq = float(self.cinfo.get_lq())
					tpl = float(self.cinfo.get_tpl())
					#print rssi, lq, tpl
					Globals.dbg.info("Updating signal", 10)
				
					rssi_perc = 50 + (rssi / 127 / 2 * 100)
					tpl_perc = 50 + (tpl / 127 / 2 * 100)
					lq_perc = lq / 255 * 100
				

					set_signal("rssi", rssi_perc, "/signal/rssi/rssi_", ".png", self.row)
					set_signal("lq", lq_perc, "/signal/lq/lq_", ".png", self.row)
					set_signal("tpl", tpl_perc, "/signal/tpl/tpl_", ".png", self.row)
				except:
					pass
				
				return 1
			else:

				self.cinfo.deinit()
				return 0
				
		return 1
				
				
def set_signal(_type, value, im_prefix, im_suffix, row=None, address=None):
	
	if row == None:
		rows = Globals.main_list.get_conditional(address=address)
		iter = Globals.main_list.get_iter(rows[0])
	else:
		if type(row) == gtk.TreeIter:
			iter = row
		else:
			iter = Globals.main_list.get_iter(row)
	
	
	data = Globals.main_list.get(iter, _type)

	
	if round(data[_type],-1) != round(value, -1):
		
		img = gen_img_path(value, Globals.IMAGE_DIR + im_prefix, im_suffix)
		pb = gtk.gdk.pixbuf_new_from_file(img)
		Globals.dbg.info("Setting %s pixbuf" % _type+"_pb", 9)
		
		Globals.main_list.set(iter, **{_type+"_pb":pb})
	
	Globals.dbg.info("Setting %s value %s" % (_type, value), 10)
	
	Globals.main_list.set(iter, **{_type:value})

	
def gen_img_path(value, prefix, suffix):
	im = int(round(value,-1))
	return prefix + str(im) + suffix
