#
#
# dhcpconf.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#


import os
import re
import commands
import blueman.constants as constants
import re

DHCP_CONFIG_FILE = "/etc/dhcp3/dhcpd.conf"


def ip_chk(ip_str):
   pattern = r"\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b"
   r = re.match(pattern, ip_str)
   if r:
      return r.groups(1)
   else:
      return False
      
def netstatus():
	dhcp=0
	ip=0
	masq=0
	
	path = os.path.join( constants.PREFIX, "bin/blueman-ifup")
	f=open(path)
	for line in f:
		m = re.match("^MASQ=(.*)", line)
		if m:
			masq = int(m.groups(1)[0])
		
		m = re.match("^DHCP=(.*)", line)
		if m:
			dhcp = int(m.groups(1)[0])
		
		m = re.match("^IP=(.*)", line)
		if m:
			ip = m.groups(1)[0]
			
	f.close()
			
	return {"dhcp":dhcp, "ip":ip, "masq":masq}
	
def have_dhcpd():
	out = commands.getoutput("whereis dhcpd3")
	s = out.split(":")
	if len(s[1]) > 0:
		return True
	else:
		return False

class netconf:


	def __init__(self, ipaddress, allow_nat = True):
		self.allow_nat = allow_nat
		
		self.dhcp_config = ""
		self.dhcp_interfaces = ""
		
		self.dns_servers = ""
		self.ipaddress = ipaddress
		self.subnet = ""
		self.existing_subnet = ""
		
		
		self.ifup_script = ""
		
		self._dhcp_interfaces_set = False # if true we dont need to start/restart the server
		

		

		self.read_dhcp_config()
		
		if ipaddress != 0:
			self.ipsegs = re.match("(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)", self.ipaddress).groups(1)
			self.subnetip = "%s.%s.%s.0" % (self.ipsegs[0], self.ipsegs[1], self.ipsegs[2])
			self.set_dns_servers()
			self.set_subnet()


				
	def read_dhcp_config(self):
		f = open(DHCP_CONFIG_FILE, "r")
		insection = False
		for line in f:
			if line == "#### BLUEMAN AUTOMAGIC SUBNET ####\n":
				insection = True
				
			if line == "#### END BLUEMAN AUTOMAGIC SUBNET ####\n":
				insection == False
				
			if not insection:
				self.dhcp_config += line
			else:
				self.existing_subnet += line
				
		f.close()
		
	
	
	def set_dns_servers(self):
		f = open("/etc/resolv.conf", "r")
		for line in f:
			server = re.search("^nameserver (.*)", line).groups(1)[0]
			self.dns_servers += "%s, " % server;
			
		self.dns_servers = self.dns_servers.strip(", ")
		
		f.close()
		
	def set_subnet(self):
		ipsegs = self.ipsegs

		start = "%s.%s.%s.%s" %(ipsegs[0], ipsegs[1], ipsegs[2], int(ipsegs[3])+1)
		end = "%s.%s.%s.%s" %(ipsegs[0], ipsegs[1], ipsegs[2], 254)
		
		
		self.subnet = "#### BLUEMAN AUTOMAGIC SUBNET ####\n"
		self.subnet += "# Everything inside this section is destroyed after config change\n"
		self.subnet += """subnet %s netmask 255.255.255.0 {
				option domain-name-servers %s;
				option subnet-mask 255.255.255.0;
				option routers %s;
				range %s %s;
				}\n""" % (self.subnetip, self.dns_servers, self.ipaddress, start, end)
		
		self.subnet += "#### END BLUEMAN AUTOMAGIC SUBNET ####\n"
		

					
	
	def setup_dhcp(self):
		restart_required = False
		if self.subnet != self.existing_subnet:
			f = open(DHCP_CONFIG_FILE, "w")
			f.write(self.dhcp_config)
			f.write(self.subnet)
			restart_required = True
			f.close()
		
 
 		return restart_required
 		
 	def setup_ifup(self, execute = False):
 		self.generate_ifup_script()
 		path = os.path.join( constants.PREFIX, "bin/blueman-ifup")
		s = netstatus()
 		f = open( path, "w" )
 		
 		f.write(self.ifup_script)
 		
 		f.close()
 		
 		
 		if not s["masq"] and self.allow_nat and not execute:
		 	os.system("%s pan1 1" % path)

		if s["masq"] and not self.allow_nat and not execute:
		 	try:
		 		f = open("/tmp/blueman-iptbl-lock")
				rule = f.readline()
				f.close()
				os.system("rm -f /tmp/blueman-iptbl-lock")
				os.system("iptables -t nat -D POSTROUTING %s" % rule)
			except:
				pass
 			
 		
 		
 		if execute:
 			if os.path.exists("/tmp/blueman-dh-lock"):
 				os.system("cat /tmp/blueman-dh-lock | xargs kill")
 				os.system("rm -f /tmp/blueman-dh-lock")
 			
 			if os.path.exists("/tmp/blueman-iptbl-lock"):
 				f = open("/tmp/blueman-iptbl-lock")
 				rule = f.readline()
 				f.close()
 				os.system("rm -f /tmp/blueman-iptbl-lock")
 				os.system("iptables -t nat -D POSTROUTING %s" % rule)
 			
 			os.system("%s pan1" % path)
 			
 	def unsetup_dhcp(self):
		if os.path.exists("/tmp/blueman-dh-lock"):
			os.system("cat /tmp/blueman-dh-lock | xargs kill")
			os.system("rm -f /tmp/blueman-dh-lock")	
			
		if os.path.exists("/tmp/blueman-iptbl-lock"):
			f = open("/tmp/blueman-iptbl-lock")
			rule = f.readline()
			f.close()
			os.system("rm -f /tmp/blueman-iptbl-lock")
			os.system("iptables -t nat -D POSTROUTING %s" % rule)
			
		f = open(DHCP_CONFIG_FILE, "w")
		f.write(self.dhcp_config)
		f.close()
 	
 	def unsetup_ifup(self):
 		path = os.path.join( constants.PREFIX, "bin/blueman-ifup")
 		f = open(path, "w");
 		f.write("#!/bin/bash\n")
 		f.write("MASQ=0\nIP=0\nDHCP=0\n\nif [ \"$1\" == \"pan0\" ]; then\n\tavahi-autoipd $1\nfi""")
 		f.close()
 		

				
	def generate_ifup_script(self):
		
		
		self.ifup_script = """#!/bin/bash
#this script is generated automatically by blueman, for your own good, do not edit :)
MASQ=%s
IP=%s
DHCP=1

if [ "$1" == "pan0" ]; then
	avahi-autoipd $1
fi

if [ "$1" == "pan1" ]; then
	if ! [ "$2" == "1" ]; then
		ifconfig pan1 %s netmask 255.255.255.0 up
	fi
	if ! [ -a /tmp/blueman-iptbl-lock ]; then
		
		if [ "$MASQ" == "1" ]; then
			echo 1 > /proc/sys/net/ipv4/ip_forward
			rule="-s %s/255.255.255.0 -j MASQUERADE"
			iptables -t nat -A POSTROUTING $rule
			echo $rule > /tmp/blueman-iptbl-lock
		fi
	fi
	if ! [ -a /tmp/blueman-dh-lock ]; then
		dhcpd3 pan1
		PID=`pidof -s -o %%PPID -x dhcpd3`
		echo $PID > /tmp/blueman-dh-lock
	fi
	
fi

""" % ( int(self.allow_nat), self.ipaddress, self.ipaddress, self.subnetip)



