#
#
# manager.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
import blueman.Globals as Globals
import dbus
import gobject
from blueman.Services.bluetooth.Transfer.server import server
from blueman.Services.bluetooth.Transfer.session import session

from blueman.Main.dbus_signal_handler import dbus_signal_handler
from blueman.Main.signal_handler import signal_handler

import blueman.Globals as Globals

class manager(gobject.GObject):
	__gsignals__ = {
		'manager-closed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
		'server-created': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
		'server-destroyed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
	
	}
	def __init__(self):
		gobject.GObject.__init__(self)
		
		self.bus = dbus.SessionBus()
		
		
		self.obj = self.bus.get_object('org.openobex', '/org/openobex')
		self.manager = dbus.Interface(self.obj, 'org.openobex.Manager')
		
		self.signals = dbus_signal_handler(self.manager)
		
		self.signals.connect('SessionCreated', self.session_created)
		self.signals.connect('SessionRemoved', self.session_removed)
		
		self.servers = {}
		self.server_ids = []
		self.gsignals = {}
		
		self.NOW = self.bus.watch_name_owner('org.openobex', self.name_owner_changed)
		

		if Globals.Settings.trans_ftp_autostart:
			path = Globals.Settings.trans_ftp_path
			allow_del = Globals.Settings.trans_ftp_allowdelete

			self.start_server("ftp", path, allow_write=allow_del)

			
		if Globals.Settings.trans_opp_autostart:
			path = Globals.Settings.trans_opp_path
			allow_del = Globals.Settings.trans_opp_allowdelete
			self.start_server("opp", path, allow_write=allow_del)

		
	#def __del__(self):
	#	print "deleting manager"
		
	def __deinit__(self, graceful=True):
		
		self.NOW.cancel()
		
		if graceful:
			self.signals.deinit()

		for i in range(len(self.server_ids)):
				self.servers[self.server_ids[i]].__deinit__(graceful)
				print "--ser deinit--"
		
		del self.signals, self.manager, self.obj, self.bus
		
		self.emit("manager-closed")
		
	
	def name_owner_changed(self, arg):
		#print "name owner chngd %s" % arg
		if arg == "":
			self.__deinit__(False)
		
	
	def get_interface(self, path, interface):
		self.obj = self.bus.get_object('org.openobex', path)
		return dbus.Interface(self.obj, interface)
		
	
	def start_server(self, profile, path, address="00:00:00:00:00:00", allow_write=False):
		if profile == "ftp" or profile == "opp":
	
			if profile == "ftp":
				require_pairing = True 
			else:
				require_pairing = False
			try:
				obj_path = self.manager.CreateBluetoothServer(address, profile, require_pairing)

			
				print "starting %s %s" % (require_pairing,allow_write)
				self.servers[profile] = server(obj_path, path, profile, allow_write, require_pairing)
				self.gsignals[profile] = self.servers[profile].connect("server-closed", self.server_closed, profile)
				self.server_ids.append(profile)
				self.emit("server-created", profile)
			except:
				Globals.dbg.warn("Failed to autostart %s service" % profile)
	
	def close_server(self, profile):
		self.servers[profile].close()

		
	def stop_server(self, profile):
		self.servers[profile].stop()
		self.servers[profile].close()
		

	def is_server_running(self, profile):
		try:
			return self.servers[profile].is_started()
		except:
			return None
	
	def session_created(self, path):
		pass
		
	def session_removed(self, path):
		pass
		
	def server_closed(self, server, profile):
		#print "deleting %s" % profile
		del self.servers[profile]
		server.disconnect(self.gsignals[profile])
		del self.gsignals[profile]
		self.emit("server-destroyed", profile)
		
	def get_server(self, profile):
		try:
			return self.servers[profile]
		except:
			return None
			
	def get_servers(self):
		return self.servers
