#
#
# server.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import dbus
import gobject

from blueman.Services.bluetooth.Transfer.server_session import server_session
from blueman.Main.dbus_signal_handler import dbus_signal_handler


class server( gobject.GObject):
	__gsignals__ = {
		'session-started': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, gobject.TYPE_INT,)),
		'session-ended': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, gobject.TYPE_INT,)),
		'server-started': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
		'server-stopped': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
		'server-closed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
		'error-occurred': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, gobject.TYPE_STRING)),
	
	}
	
	
	def __init__(self, obj_path, path, profile, allow_write=False, require_pairing=False ):
		gobject.GObject.__init__(self)

		self.closed = False
		self.deinitialized = False

		self.sessions = {}
		self.ses_addrs = {}
		
		self.profile = profile
		#todo: check if path exists
		
		self.path = path
		self.allow_write = allow_write
		self.require_pairing = require_pairing

		self.server = self.get_interface(obj_path, 'org.openobex.Server')
		self.signals = dbus_signal_handler(self.server)
		
		self.signals.connect("SessionCreated", self.server_session_created)
		self.signals.connect("SessionRemoved", self.server_session_removed)
		self.signals.connect("Started", self.server_started)
		self.signals.connect("Stopped", self.server_stopped)
		self.signals.connect("ErrorOccured", self.error_occured)
		self.signals.connect("Closed", self.server_closed)
		
		self.start()
			

	def __deinit__(self, graceful=True):
		if not self.deinitialized:
			print self.sessions
			for k,v in self.sessions.iteritems():
				v.__deinit__(graceful)
				
			del self.sessions
		
			if not graceful:
				del self.signals, self.server
				self.emit("server-closed")
				return

			
			if not self.closed:
				self.close()


			self.signals.deinit()
			self.deinitialized = True
			
			self.emit("server-closed")
		#todo: deinit sessions
	
	
	def get_interface(self, path, interface):
		bus = dbus.SessionBus()
		obj = bus.get_object('org.openobex', path)
		return dbus.Interface(obj, interface)	
	
	def get_session_by_addr(self, addr):
		if addr in self.ses_addrs:
			id = self.ses_addrs[addr]
			return self.sessions[id]
		else:
			return None
		
	def get_session_by_id(self, id):
		if id in self.sessions:
			return self.sessions[id]
		else:
			return None
	
	
	def remove_session(self, path):
		num = int(path[len(path)-1:])
		
		for k, v in self.ses_addrs.iteritems():
			if v == num:
				del self.ses_addrs[k]
				self.emit("session-ended", k, num)
				break
		#print "remove session"		
		self.sessions[num].__deinit__()
		del self.sessions[num]
		
		#print self.sessions
		#print self.ses_addrs
				
	
	def add_session(self, path):
		num = int(path[len(path)-1:])
		ses = server_session(self.server, path, self.profile)
		addr = ses.get_remote_address()
		
		self.sessions[num] = ses
		self.ses_addrs[addr] = num
		
		self.emit("session-started", addr, num)
	
	def start(self):	

		self.server.Start(self.path, self.allow_write, False)
		
	def stop(self):
		self.server.Stop()
		
	
	def close(self):
		self.server.Close()
		
		
	def is_started(self):
		return self.server.IsStarted()
		
	
	def server_session_created(self, path):
		Globals.dbg.info("Server session created %s" % path, 5)
		self.add_session(path)
			
	def server_session_removed(self, path):
		Globals.dbg.info("Server session removed %s" % path, 5)
		self.remove_session(path)
		
	def server_started(self):
		Globals.dbg.info("Server Started", 5)
		self.emit("server-started")

		
	def server_stopped(self):
		Globals.dbg.info("Server Stopped", 5)
		self.emit("server-stopped")
		
	def server_closed(self):
		self.closed = True
		self.__deinit__()
		
	def error_occured(self, name, message):
		self.emit("error-ocurred", name, message)

		
	
	def transfer_count(self):
		num = 0
		for k,v in self.sessions.iteritems():
			if v.transfer != None:
				num += 1
				
		return num
		
	
		
		
		
		
		
		
