#
#
# session.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#from blueman.Services.bluetooth.Transfer.manager import manager
import blueman.Globals as Globals

import gobject
import dbus

from blueman.Main.dbus_signal_handler import dbus_signal_handler

class session(gobject.GObject):
	
	__gsignals__ = {
		'cancelled': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
		'disconnected': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
		'transfer-started': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, 
										 gobject.TYPE_STRING,
										 gobject.TYPE_INT64,)),
		'transfer-progress': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_INT64,)),
		'transfer-completed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
		'error-occurred': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,
									       gobject.TYPE_STRING,))
	}


	def __init__(self, path, interface):
		#manager.__init__(self)
		gobject.GObject.__init__(self)
		
		self.path = path
		self.id = int(path[len(path)-1:])
		
		self.session = self.get_interface(path, interface)
		
		#if Globals.Settings.trans_opp_trustedonly:
		#	addr = self.get_remote_address()
		#	if not Globals.adapter.IsTrusted(addr):
		#		self.ses_disconnect()
		
		
				
		
		self.signals = dbus_signal_handler(self.session)
		
		self.signals.connect("Cancelled", self.cancelled)
		self.signals.connect("Disconnected", self.disconnected)
		self.signals.connect("TransferStarted", self.transfer_started)
		self.signals.connect("TransferProgress", self.transfer_progress)
		self.signals.connect("TransferCompleted", self.transfer_completed)
		self.signals.connect("ErrorOccurred", self.error_occurred)
		
		self.transfer = None

	def __deinit__(self, graceful=True):
		print "--deinit-- " + str(self.signals)
		if not graceful:
			del self.signals, self.session
		else:
			self.signals.deinit()
			del self.signals, self.session


			
	def get_interface(self, path, interface):
		bus = dbus.SessionBus()
		obj = bus.get_object('org.openobex', path)
		return dbus.Interface(obj, interface)		
	
	def get_remote_address(self):
		_dict = self.manager.GetSessionInfo(self.path)
		if "BluetoothAddress" in _dict:
			return _dict["BluetoothAddress"]
		else:
			return None
	
	def ses_disconnect(self):
		self.session.Disconnect()
	
	def get_transfer_info(self):
		return self.session.GetTransferInfo()
	
	def close(self):
		self.session.Close()
	
	def cancel(self):
		self.session.Cancel()
	
	
	def disconnected(self):
		Globals.dbg.info("'disconnected' in %s" % self.path, 5)
		self.Transfer = None
		self.emit("disconnected")
	
	def cancelled(self):
		Globals.dbg.info("'cancelled' in %s" % self.path, 5)
		self.emit("cancelled")
		self.Transfer = None
	
	def transfer_started(self, filename, local_path, total_bytes):
		

		
		
		Globals.dbg.info("'transfer_started' in %s (%s, %s, %s)" % (self.path, filename, local_path, total_bytes), 5)
		self.transfer = { "filename" : filename, "path": local_path, "size" : total_bytes }

		self.emit("transfer-started", filename, local_path, total_bytes)

		
	def transfer_progress(self, bytes):
		Globals.dbg.info("'transfer_progress' in %s (%s)" % (self.path, bytes), 10)
		self.emit("transfer-progress", bytes)
	
	def transfer_completed(self):
		Globals.dbg.info("'transfer_completed' in %s" % self.path, 5)
		self.Transfer = None
		self.emit("transfer-completed")
	
	def error_occurred(self, name, msg):
		Globals.dbg.warn("'error_occured' in %s (%s, %s)" % (self.path, name, msg), 5)
		self.emit("error-occurred", name, msg)
		self.ses_disconnect()
		self.Transfer = None
		
	def __del__(self):
		print "deleting session"
