#
#
# network.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import blueman.Globals as Globals
import gtk
from blueman.Gui.device_selector_combobox import device_selector_combobox
from blueman.Gui.generic_list import generic_list
from blueman.Services.gui.generic import generic

from blueman.Main.netconf import netstatus, have_dhcpd, ip_chk
from blueman.Main.polkit_auth import polkit_auth
from blueman.Main.mechanism import mechanism

_ = Globals._

class network(generic):

	def __init__(self):
            	generic.__init__(self, "network")
            	
            	self.server = None
            	self.busy = None

		
		self.b_connect = self.xml.get_widget("b_connect")
		self.b_disconnect = self.xml.get_widget("b_disconnect")
		self.b_rem_conn = self.xml.get_widget("b_remove_connection")
		
		self.b_create = self.xml.get_widget("b_create")
		self.lb_create = self.xml.get_widget("lb_create")
		self.cb_address = self.xml.get_widget("cb_address")
		
		device_selector_combobox(self.cb_address, self.on_address_changed)
		
		self.cb_type = self.xml.get_widget("cb_type")

		self.cb_type.set_active(0)
		
		self.b_gn = self.xml.get_widget("b_gn")
		self.b_nap = self.xml.get_widget("b_nap")
		
		self.i_gn = self.xml.get_widget("i_gn")
		self.i_nap = self.xml.get_widget("i_nap")
		
		self.cb_nat = self.xml.get_widget("cb_nat")
		self.e_ip = self.xml.get_widget("e_ip")
		self.l_ip = self.xml.get_widget("l_ip")
		self.cb_dhcp = self.xml.get_widget("cb_dhcp")
		self.vbox_dhcp = self.xml.get_widget("vbox_dhcp")
		self.b_apply = self.xml.get_widget("b_apply")
		
		self.al_apply = self.xml.get_widget("al_apply")
		
		hb_notfound = self.xml.get_widget("hb_notfound")
		
		self.auth = polkit_auth()
		
		if not have_dhcpd():
			self.al_apply.hide()
			self.cb_dhcp.set_sensitive(False)
			hb_notfound.show()

		
		if self.service.get_server("nap").IsEnabled() :
			self.b_nap.set_active(True)
			self.i_nap.set_from_stock(gtk.STOCK_YES, 4)
		
		if self.service.get_server("gn").IsEnabled():
			self.b_gn.set_active(True)
			self.i_gn.set_from_stock(gtk.STOCK_YES, 4)
		
		self.b_apply.set_sensitive(False)
		nstatus = netstatus()
		self.nstatus = nstatus
		if nstatus["dhcp"]:
			self.cb_dhcp.set_active(True)
			self.set_dhcp(True)
		if nstatus["ip"] != "0":
			self.e_ip.set_text(nstatus["ip"])

		if nstatus["masq"]:
			self.cb_nat.set_active(True)
		
		
		
		######## list #########
		self.conn_list = self.xml.get_widget("tv_conn");

		data = [
			["icon", str, gtk.CellRendererPixbuf(), {"stock_id":0}, None],
			["name", str, gtk.CellRendererText(), {"markup":1}, _("Name")],
			["address", str, gtk.CellRendererText(), {"text":2}, _("Address")],
			["interface", str, gtk.CellRendererText(), {"text":3}, _("Interface")],
			["path", str]
		]
		
		sw = self.xml.get_widget("sw")
		self.list = generic_list(data)
		self.list.connect("cursor-changed", self.on_list_select)
		self.list.show()
		sw.add(self.list)

		
		self.populate_conn_list()
		

		self.signals.connect("connect", self.on_connect)
		self.signals.connect("disconnect", self.on_disconnect)
		self.signals.connect("connection-removed", self.on_connection_removed)
		self.signals.connect("connection-created", self.on_connection_created)
		#self.signals.connect("enabled", self.on_server_enabled)
		#self.signals.connect("disabled", self.on_server_disabled)
		
	       
		self.xml.signal_autoconnect(self)
		
		self.dialog.show()
		
		
	def set_dhcp(self, state):
		self.vbox_dhcp.set_sensitive(state)

	def check_config(self):
		x = True
		x = x and self.e_ip.get_text() == self.nstatus["ip"]
		x = x and self.cb_nat.get_active() == self.nstatus["masq"]
		x = x and self.cb_dhcp.get_active() == self.nstatus["dhcp"]
			
		if x:
			self.b_apply.set_sensitive(False)
		else:
			self.b_apply.set_sensitive(True)
	

	def on_e_ip_changed(self, widget):
		self.check_config()
		
	def on_cb_nat_toggled(self, widget):
		self.check_config()
			
		
	def on_dhcp_toggled(self, widget):
		self.check_config()
		
		if not widget.get_active():
			self.set_dhcp(False)
		else:
			self.set_dhcp(True)
			
		
			
	def on_dhcp_apply_clicked(self, widget):
		self.set_notice(None)
		self.auth.obtain_authorization_async(widget, "org.blueman.network.setup", self.on_auth_reply, self.on_auth_err)
		
	def on_auth_reply(self, status):
		if self.auth.is_authorized("org.blueman.network.setup") or status:
			m = mechanism()
			
			if not self.cb_dhcp.get_active():
				m.NetworkSetup("0", 0) #deinitialize the settings
			else:
				ip = self.e_ip.get_text()
				ipsegs = ip_chk(ip)
				if ipsegs:					
					m.NetworkSetup(ip, self.cb_nat.get_active())
					self.nstatus = netstatus()
					self.b_apply.set_sensitive(False)
				else:
					self.set_notice("gtk-dialog-error", _("<b>Invalid IP address entered</b>"))
					
			del m

		
	def on_auth_err(self, *args):
		pass
		
	def on_b_gn_toggled(self, togglebutton):
		server = self.service.get_server("gn")
		status = server.IsEnabled()
		if not status:
			server.Enable()
			self.i_gn.set_from_stock(gtk.STOCK_YES, 4)
		else:
			server.Disable()
			self.i_gn.set_from_stock(gtk.STOCK_NO, 4)
		
	def on_b_nap_toggled(self, togglebutton):
		server = self.service.get_server("nap")
		
		status = server.IsEnabled()
		if not status:
			server.Enable()
			self.i_nap.set_from_stock(gtk.STOCK_YES, 4)
			Globals.Settings.net_nap_autostart = 1
		else:
			server.Disable()
			self.i_nap.set_from_stock(gtk.STOCK_NO, 4)
			Globals.Settings.net_nap_autostart = 0
			
		Globals.Settings.save()
		
        
	def populate_conn_list(self):
		self.list.clear()
		self.server = None
		self.b_rem_conn.set_sensitive(False)
		conns = self.service.get_connections()
		for i in range(len(conns)):
			name = conns[i][0].GetName()
			addr = conns[i][0].GetAddress()
			path = conns[i][1]
			
			if conns[i][0].IsConnected():
				conn_pb = "gtk-connect"
				interface= conns[i][0].GetInterface()
			else:
				conn_pb = None
				interface = "N/A"
			
			
			self.list.add(name="<b>"+name+"</b>", address=addr, path=path, icon=conn_pb, interface=interface)

    
    	def on_list_select(self, list):
		row = self.list.get(self.list.selected(), "path")
		self.server = self.service.get_connection(row["path"])
		self.server.path=row["path"]
		self.reset_buttons()
      
    
    	def reset_buttons(self):
		self.b_rem_conn.set_sensitive(True)
		if not self.busy:
			self.set_notice(None)
		if self.server.IsConnected():
	    		self.b_connect.set_sensitive(False)
	    		self.b_disconnect.set_sensitive(True)
       		else:
	    		self.b_connect.set_sensitive(True)
	    		self.b_disconnect.set_sensitive(False)
	    
    	def disable_buttons(self):
		self.b_connect.set_sensitive(False)
		self.b_disconnect.set_sensitive(False)
		self.b_rem_conn.set_sensitive(False)
        	
        def disable_create(self):
        	self.cb_type.set_sensitive(False)
        	self.b_create.set_sensitive(False)
        	#self.e_host.set_sensitive(False)
        	
        def enable_create(self):
        	self.cb_type.set_sensitive(True)
        	self.b_create.set_sensitive(True)
        	#self.e_host.set_sensitive(True)        	
    
    	## -- Connect -- ##
    	def on_b_connect_clicked(self, widget):
        	self.disable_buttons()
        	self.set_notice("gtk-info", _("<b>Connecting...</b>"))

        	self.server.Connect(error_handler=self.connect_err, reply_handler=self.connect_reply)
    	
    	
    	def connect_err(self, err):
    		try:
	    		row = self.list.get(self.list.selected(), "path")
	    		if row["path"] == self.server.path:			
				self.reset_buttons()
		
			self.set_notice("gtk-dialog-error", "<b>"+str(err).split(": ")[1]+"</b>")
		except:
			pass
    		
    	def connect_reply(self, path):
    		self.set_notice(None)
    	
    	def on_connect(self, service, path):
		self.populate_conn_list()
		

        
	## -- Connection/server stuff -- ##
        def on_b_remove_connection_clicked(self, widget):
        	row = self.list.get(self.list.selected(), "path")
          	self.service.remove_connection(row["path"])

	def on_connection_removed(self, connection, path):
		self.populate_conn_list()
	
	
	def on_b_create_clicked(self, widget):
		self.disable_create()


		addr = self.cb_address.get_active_text()
		id=self.cb_type.get_active()
		if id == 0:
			type="NAP"
		else:
			type="GN"
		
		self.service.create_connection(addr, type, reply_handler=self.create_reply, error_handler=self.create_err)
		self.set_notice("gtk-info", _("<b>Creating Server...</b>"))
		self.busy= True
		
		Globals.dbg.info("Creating Server (%s)" % addr, 4)
	
	
	def create_reply(self, *args):
		self.busy = False
		self.set_notice(None)
		
		
		
	def create_err(self, err):
		try:
			self.set_notice("gtk-dialog-error", "<b>" + str(err).split(": ")[1] + "</b>")
			self.enable_create()
			self.busy = False
		
			Globals.dbg.warn("Failed To Create Server. Reason: %s" % err, 4)
		except:
			pass
	
	def on_connection_created(self, connection, path):
		self.populate_conn_list()  
		self.enable_create()
		
		Globals.dbg.info("Server Created (DBus path - %s)" % path, 4)
	
			
	
    	
    	## -- Disconnect -- ##
     	def on_b_disconnect_clicked(self, widget):
        	self.disable_buttons()
        	self.server.Disconnect()   	
    	
    	
    	def on_disconnect(self, service, path):
    		row = self.list.get(self.list.selected(), "path")
    		if row:
	    		if row["path"] == self.server.path:
				self.reset_buttons()

		rows = self.list.get_conditional(path=path)
		self.list.set(rows[0], icon=None, interface="N/A")
		
	def on_address_changed(self, state):
		
		if state:
			self.b_create.set_sensitive(True)
		else:
			self.b_create.set_sensitive(False)

		
	## -- server stuff -- ##
	
	def on_b_start_clicked(self, widget):
		if self.rb_nap.get_active():
			serv = self.service.get_server("nap")
		else:
			serv = self.service.get_server("gn")
		
		serv.Enable()



