#
#
# transfer.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
import blueman.Globals as Globals
import gtk
import gobject
import os

from blueman.Gui.device_selector_combobox import device_selector_combobox

from blueman.Services.gui.generic import generic

from blueman.Gui.file_chooser import file_chooser

from blueman.Main.signal_handler import signal_handler

_ = Globals._

class transfer(generic):

	def __init__(self):
		generic.__init__(self, "transfer")

		
		self.g_manager_signals = signal_handler(self.service.manager)
		self.g_manager_signals.connect("manager-closed", self.manager_closed)
		self.g_manager_signals.connect("server-created", self.server_created)
		self.g_manager_signals.connect("server-destroyed", self.server_destroyed)
		


		
		self.init_ftp()
		self.init_opp()
		
		self.xml.signal_autoconnect(self)

		
	def dialog_response(self, dialog, response):
		self.g_manager_signals.deinit()
		
	def on_ftp_browse_clicked(self, widget):
		filename = file_chooser(_("Select Folder")).get_folder()
		
		if filename != None:
			self.ftp_e_path.set_text(filename)
		
	def on_opp_browse_clicked(self, widget):
		filename = file_chooser(_("Select Folder")).get_folder()
		
		if filename != None:
			self.opp_e_path.set_text(filename)
	
	def init_ftp(self):
		self.ftp_start = self.xml.get_widget("ftp_start")
		self.ftp_stop = self.xml.get_widget("ftp_stop")
		self.ftp_cb_allowdelete = self.xml.get_widget("ftp_allowdelete")
		self.ftp_filechooser = self.xml.get_widget("ftp_filechooser")
		self.ftp_cb_autostart = self.xml.get_widget("ftp_autostart")
		self.ftp_cb_trustedonly = self.xml.get_widget("ftp_trustedonly")
		
		self.ftp_cb_allowdelete.set_active(Globals.Settings.trans_ftp_allowdelete)
		self.ftp_filechooser.set_filename(Globals.Settings.trans_ftp_path)
		self.ftp_cb_autostart.set_active(Globals.Settings.trans_ftp_autostart)
		self.ftp_cb_trustedonly.set_active(Globals.Settings.trans_ftp_trustedonly)
		
		self.set_ftp_buttons()
		
	def init_opp(self):
		self.opp_start = self.xml.get_widget("opp_start")
		self.opp_stop = self.xml.get_widget("opp_stop")
		self.opp_cb_allowdelete = self.xml.get_widget("opp_allowdelete")
		self.opp_filechooser = self.xml.get_widget("opp_filechooser")
		self.opp_cb_autostart = self.xml.get_widget("opp_autostart")
		self.opp_cb_trustedonly = self.xml.get_widget("opp_trustedonly")
		
		self.opp_cb_allowdelete.set_active(Globals.Settings.trans_opp_allowdelete)
		self.opp_filechooser.set_filename(Globals.Settings.trans_opp_path)
		self.opp_cb_autostart.set_active(Globals.Settings.trans_opp_autostart)
		self.opp_cb_trustedonly.set_active(Globals.Settings.trans_opp_trustedonly)
		
		self.set_opp_buttons()		
			
	def on_ftp_stop(self, widget):
		self.service.manager.stop_server("ftp");
		self.set_ftp_buttons()
		
	def on_opp_stop(self, widget):
		self.service.manager.stop_server("opp");
		self.set_opp_buttons()	
	
	def on_ftp_start(self, widget):
		self.save_settings()
		if not self.status("ftp"):

			self.service.manager.start_server("ftp", self.ftp_filechooser.get_filename(), allow_write=self.ftp_cb_allowdelete.get_active())
			self.save_settings()

		
		self.set_ftp_buttons()
	
	
	def on_opp_start(self, widget):
		self.save_settings()
		if not self.status("opp"):

			self.service.manager.start_server("opp", self.opp_filechooser.get_filename(), allow_write=self.ftp_cb_allowdelete.get_active())
			self.save_settings()

		
		self.set_opp_buttons()			

		
	
	def validate_opp(self):
		if not os.path.exists(self.opp_e_path.get_text()) or not os.access(self.opp_e_path.get_text(), os.W_OK) or not os.path.isdir(self.opp_e_path.get_text()):
			return False
		else:
			return True
			
	def save_settings(self):
		Globals.Settings.trans_ftp_allowdelete = self.ftp_cb_allowdelete.get_active()
		Globals.Settings.trans_ftp_path = self.ftp_filechooser.get_filename()
		Globals.Settings.trans_ftp_autostart = self.ftp_cb_autostart.get_active()
		Globals.Settings.trans_ftp_trustedonly = self.ftp_cb_trustedonly.get_active()
		
		Globals.Settings.trans_opp_allowdelete = self.opp_cb_allowdelete.get_active()
		Globals.Settings.trans_opp_path = self.opp_filechooser.get_filename()
		Globals.Settings.trans_opp_autostart = self.opp_cb_autostart.get_active()
		Globals.Settings.trans_opp_trustedonly = self.opp_cb_trustedonly.get_active()
		
		Globals.Settings.save()	
	

		
	###### MANAGER SIGNALS #####
	def manager_closed(self, manager):
		print "g manager closed"
		
	def server_created(self, manager, profile):
		print "g server_created %s" % profile
		#server = self.service.manager.get_server(profile)
		#handler = signal_handler(server)
		#self.g_server_signals[server] = handler
		#handler.connect("session-started", self.session_created)
		#handler.connect("session-ended", self.session_ended)
		#handler.connect("server-started", self.server_started)
		#handler.connect("server-stopped", self.server_stopped)

		
		#self.add_server(profile)
		
		
	def server_destroyed(self, manager, profile):
		print "g server_destroyed %s" % profile

	############################

	
	def set_ftp_buttons(self):
		if not self.status("ftp"):
			self.ftp_start.set_sensitive(True)
			self.ftp_stop.set_sensitive(False)
		else:
			self.ftp_start.set_sensitive(False)
			self.ftp_stop.set_sensitive(True)
			
	def set_opp_buttons(self):
		if not self.status("opp"):
			self.opp_start.set_sensitive(True)
			self.opp_stop.set_sensitive(False)
		else:
			self.opp_start.set_sensitive(False)
			self.opp_stop.set_sensitive(True)
		
	def status(self, profile):
		x = self.service.manager.is_server_running(profile)
		if x == None or x == False:
			return False
		else:
			return True
	
	def __destroy__(self):
		self.save_settings()
		self.g_manager_signals.deinit()
		generic.__destroy__(self)
		
	

			
	
	

			
			
			
			

	
		
		
		
		
