#
#
# service_manager.py
# (c) 2007 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#


from blueman.Gui.generic_dialog import generic_dialog
import blueman.Globals as Globals

from blueman.Services.service_manager.audio import audio
from blueman.Services.service_manager.network import network
from blueman.Services.service_manager.input import input
from blueman.Services.service_manager.serial import serial
from blueman.Services.service_manager.transfer import transfer



class manager(generic_dialog):
	running = False
	def __init__(self):
		if manager.running:
			return
		generic_dialog.__init__(self, "services.glade", "services")

		manager.running = True
		
		self.services = []
		
		self.signal = Globals.Manager.connect("adapter-detached", self.adapter_detached)
		self.signal2 = Globals.Manager.connect("adapter-attached", self.adapter_attached)
		
		ls = Globals.Manager.ListAdapters()
		if len(ls) > 0:
			self.init_services()

	def init_services(self):
		self.services.append( audio(self.xml) )
		self.services.append( network(self.xml) )
		self.services.append( input(self.xml) )
		self.services.append( serial(self.xml) )
		self.services.append( transfer(self.xml) )


	def adapter_attached(self, obj, path):
		ls = Globals.Manager.ListAdapters()
		if len(ls) == 1:
			self.init_services()

	def adapter_detached(self, obj, path):
		ls = Globals.Manager.ListAdapters()
		if len(ls) == 0:
			for i in range(len(self.services)):
				
				if self.services[i]._instance != None:
					self.services[i]._g_inst_signal.deinit()
					self.services[i]._instance.__destroy__()
			
					self.services[i]._instance = None
					self.services[i]._g_inst_signal = None
				
				self.services[i].set_state(-1)
				self.services[i].deinit()
				

				
			del self.services
			self.services = []

		

	def dialog_response(self, dialog, response):
		for i in range(len(self.services)):
			self.services[i].deinit()
			
		self.services = []
		
		Globals.Manager.disconnect(self.signal)
		Globals.Manager.disconnect(self.signal2)
		
		manager.running = False
		
		return False


